Renvs= new.env()
##this doesn't seem to work anyway...
#if(getRversion() >= "2.15.1") globalVariables(".lib.loc")

##' switchTo
##'
##' Switch to a different computing environment (set of installed R packages
##' and library location paths for new pkg installs)
##'
##' If switchr does not now about the specified computing environment, a new one
##' will be created via installCompEnv. This includes
##' creating a directory under the switchr base directory and installing
##' packages into it. See \code{installCompEnv} for more details.
##'
##' @param name The name associated (or to associate) with the computing
##' environment.
##' @param seed The seed, indicating packages to install into a newly created
##' package library
##' No effect if the library already exists
##' @param reverting Indicates whether we are reverting to the environment in
##' use before the current one. Typically not set directly by the user.
##' @param ignoreRVersion Should the R version in use be ignored when checking
##' for existing computing environmeSnts. This is experimental.
##' @param ... Passed directly to \code{installCompEnv} if an existing
##' computing environment is not found.
##' @details This function has the side effect of unloading all loaded
##' packages (other than base packages, GRAN or GRANBAse,  switchr itself, and
##' switchr's dependencies) and the associated DLLs. It also changes the library
##' location R will use to search for packages, e.g. when you call
##' \code{library}.
##'
##' This means you will have to reinstall packages after switching, which is
##' important and intended (e.g. when switching to using Bioc devel from Bioc
##' release).
##'
##'
##' @return Invisibly returns the SwitchrCtx object representing the new
##' computing environment
##'
##' @examples
##' \dontrun{
##' switchTo("mynewlibrary")
##' switchBack()
##' 
##' fdman = GithubManifest("gmbecker/fastdigest")
##' switchTo("fastdigestlib", seed = fdman)
##' }
##' @export
##' @docType methods
##' @rdname switchTo
setGeneric("switchTo", function(name, seed = NULL, reverting = FALSE,
                                ignoreRVersion = FALSE,  ...)
           standardGeneric("switchTo"))

##' @rdname switchTo
##' @aliases switchTo,character,character
setMethod("switchTo", c(name = "character", seed = "character"),
          function(name, seed, reverting = FALSE, ignoreRVersion = FALSE, ...) {
        
    
    ## At this point seed is guaranteed to be a repo url
    
    if(ignoreRVersion)
        rvers = NULL
    else
        rvers = paste(R.version$major, R.version$minor, sep=".")

    cenv = findCompEnv(name = name, rvers = rvers)

    if(is.null(cenv)) {
            chtype = getStringType(seed)
            if(chtype == "file") {
                seed = readLines(seed)
                chtype = getStringType(seed)
            }
            
            if(chtype == "sessioninfo") {
                ## we have session info output
                ##XXX need to make sure double use of ... is safe!
                seed2 = makeSeedMan(parseSessionInfoString(seed))
                sr = lazyRepo(seed2, ...)
                
                
                seed = if(grepl("file://", sr)) sr else makeFileURL(sr)
                seed = gsub("(/|\\\\)src(/|\\\\)contrib.*", "", seed)
                chtype = "repourl"
                
            } else if (chtype == "manifesttxt") {
                con = textConnection(seed)
                on.exit(close(con))
                seed2 = loadManifest(con)
                close(con)
                on.exit(NULL)
                
                seed = lazyRepo(seed2, ...)
            } else if(grepl("(repo|contrib)", chtype)) {
                seed = repoFromString(seed, chtype)
                chtype = "repourl"
            }
            
            if(chtype != "repourl") {
                stop("We should have a repository by this point. This shouldn't happen. Contact the maintainers")
            }
            
            cenv = makeLibraryCtx(name = name, seed = seed, ...)
        } else {
            message(sprintf("Library %s already exists. Ignoring seed and switching to existing library"), name)
        }
    if(!is.null(cenv))
        ##        switchTo(name = name, seed = cenv)
        switchTo(name = cenv)
    else
        stop("unable to switch to computing environment")
})

repoFromString = function(str, type) {
    switch(type,
           repodir = makeFileURL(str),
           contribdir = makeFileURL(gsub("/(src|bin/windows|bin/macosx|bin/macos).*", "", str)),
           repourl = str,
           contriburl = gsub("/(src|bin/windows|bin/macosx|bin/macos).*", "", str))
}
           

##' @rdname switchTo
##' @aliases switchTo,character,SwitchrCtx

setMethod("switchTo", c(name = "character", seed= "SwitchrCtx"),
          function(name, seed, reverting = FALSE, ignoreRVersion = FALSE,...) {
              
              if(ignoreRVersion)
                  rvers = NULL
              else
                  rvers = paste(R.version$major, R.version$minor, sep=".")
              exsting = findCompEnv(name = name, rvers = rvers)
              if(!is.null(exsting)) {
                  warning("A switchr context with that name already exists")
                  switchTo(exsting)
              }
              cenv = makeLibraryCtx(name = name, seed = NULL,
                  exclude.site = seed@exclude.site,
                  ...)
              
              ## copy existing library contents to the new one
              dirs = list.dirs(file.path(switchrBaseDir(), seed@name), recursive = FALSE)
            ##  dests = file.path(library_paths(cenv)[1], basename(dirs))
    ##          dir.create(dests)
     ##         mapply(file.copy,dirs, dests, recursive=TRUE)
              file.copy(dirs, library_paths(cenv)[1],
                        recursive = TRUE, overwrite = FALSE)

              cenv = update_pkgs_list(cenv)
              switchTo(cenv)
          })

##' @rdname switchTo
##' @aliases switchTo,character,missing

setMethod("switchTo", c("character", "missing"),
          function(name, seed, reverting = FALSE, ignoreRVersion = FALSE,...) {
              
    if(ignoreRVersion)
        rvers = NULL
    else
        rvers = paste(R.version$major, R.version$minor, sep=".")

    cenv = findCompEnv(name = name, rvers = rvers)

    if(is.null(cenv))
        cenv = makeLibraryCtx(name = name, ...)


    if(!is.null(cenv))
        ##        switchTo(name = name, seed = cenv)
        switchTo(name = cenv)
    else
        stop("unable to switch to computing environment")
})


getStringType = function(str) {
    if(any(grepl("Platform:", str)))
        return("sessioninfo")
    if(grepl("^# R manifest", str[1]))
        return("manifesttxt")
    
    if(length(str) > 1)
        return(sapply(str, getStringType))

    
    if(grepl("file://", str)) {
        isfilurl = TRUE
        str = fileFromFileURL(str)
    } else
        isfilurl = FALSE
    
    if(file.exists(str)) {
        if( !file.exists(file.path(str, ".")))
            ret = "file"
        else { #if str points to a directory
            if(grepl("contrib/{0,1}$", str))
                ret = "contribdir"
            else if(file.exists(file.path(str,
                                     "src/contrib/PACKAGES")))
                ret = "repodir"
            else
                ret = "manifestdir"
            
        }
        if(!is.null(ret)) {
            if(ret != "file" && isfilurl)
                ret = gsub("dir$", "url", ret)
            return(ret)
        }
    } else if (isfilurl) { # file doesn't exist, but its a file url
        stop("file urls to non-existent files are not allowed as seeds/repos")
    }
                                     
   
    if(url.exists(paste0(str, "/PACKAGES.gz")))
        return("contriburl")
    else if (url.exists(paste0(str, "/src/contrib/PACKAGES.gz")))
        return("repourl")
    else if (url.exists(str))
        return("manifesturl")
    
    stop("Unidentifiable string:", str)
}

##' @rdname switchTo
##' @aliases switchTo,SwitchrCtx,ANY

setMethod("switchTo", c(name = "SwitchrCtx", seed = "ANY"), function(name, seed, reverting=FALSE, ...) {
        if(is.null(Renvs$stack)) {
            paths = .libPaths()
            paths = paths[!paths %in% c(.Library.site, .Library)]
            Renvs$stack = list(original = SwitchrCtx("original", paths, exclude.site=FALSE, seed = NULL))
        }

        flushSession()

        .libPaths2(library_paths(name), name@exclude.site)

         if(!reverting) {
#            attachedPkgs(Renvs$stack[[length(Renvs$stack)]]) = atched
             Renvs$stack = c(name, Renvs$stack)
        } else
            Renvs$stack = Renvs$stack[-1]
        announce(name, reverted = reverting)

        invisible(name)
    })

##' @rdname switchTo
##' @aliases switchTo,character,RepoSubset

setMethod("switchTo", c(name = "character", seed="RepoSubset"), function(name, seed = NULL,
                                             reverting = FALSE,
                                             ignoreRVersion = FALSE,
                            ...) {
    if(any(c("pkgs", "repo_name") %in% names(list(...))))
        stop("Cannot specify pkgs or repo_name when switching to a RepoSubset")
    ##seed is a RepoSubset object

    if(missing(name)) {
        name = seed@default_name
    }
        
    switchTo(seed = seed@repos, name = name, pkgs = seed@pkgs, ...)
})


##' @rdname switchTo
##' @aliases switchTo,character,PkgManifest

setMethod("switchTo", c("character", seed = "PkgManifest"),
          function(name, seed, reverting = FALSE, ignoreRVersion = FALSE, ...) {
             
              if(ignoreRVersion)
                  rvers = NULL
              else
                  rvers = paste(R.version$major, R.version$minor, sep=".")
              exsting = findCompEnv(name = name, rvers = rvers)
              if(!is.null(exsting)) {
                  warning("A switchr context with that name already exists")
                  switchTo(exsting)
              }
              cenv = makeLibraryCtx(name = name, seed = NULL,
                  ...)
              oldlp = .libPaths()
              .libPaths2(library_paths(cenv), cenv@exclude.site)
              on.exit(.libPaths2(oldlp))

              install_packages(manifest_df(seed)$name, seed, lib = library_paths(cenv)[1])
              cenv = update_pkgs_list(cenv)
              .libPaths2(oldlp)
              on.exit(NULL)
              switchTo(cenv)
          })


##' @rdname switchTo
##' @aliases switchTo,character,SessionManifest

setMethod("switchTo", c("character", seed = "SessionManifest"),
          function(name, seed, reverting = FALSE, ignoreRVersion = FALSE, ...) {
              
              if(ignoreRVersion)
                  rvers = NULL
              else
                  rvers = paste(R.version$major, R.version$minor, sep=".")
              exsting = findCompEnv(name = name, rvers = rvers)
              if(!is.null(exsting)) {
                  warning("A switchr context with that name already exists")
                  switchTo(exsting)
              }
              cenv = makeLibraryCtx(name = name, seed = NULL,
                  ...)
              

                       
              install_packages(pkgs = seed, lib = library_paths(cenv)[1])
              cenv = update_pkgs_list(cenv)
              switchTo(cenv)
          })






setGeneric("attachedPkgs<-", function(seed, value) standardGeneric("attachedPkgs<-"))
setMethod("attachedPkgs<-", "SwitchrCtx", function(seed, value) {
    seed@attached = value
    seed
})







setGeneric("announce", function(seed, reverted=FALSE) standardGeneric("announce"))

setMethod("announce", "SwitchrCtx", function(seed, reverted=FALSE) {
    message(sprintf("%s to the '%s' computing environment. %d packages are currently available.", ifelse(reverted, "Reverted", "Switched"),
                    seed@name,  nrow(seed@packages)))
    message(sprintf("Packages installed in your site library ARE %ssuppressed.", ifelse(seed@exclude.site, "", "NOT ")))
    message("To switch back to your previous environment type switchBack()")
})

setMethod("show", "SwitchrCtx", function(object) {
    message(paste(sprintf("An SwitchrCtx object defining the '%s' computing environment", object@name),
              "\n\n\t", sprintf("Primary library location(s): %s", paste(object@libpaths, collapse=";")),
              "\n\t", sprintf("Packages: %d packages installed in %d directories (including R's base library)", nrow(object@packages), length(unique(object@packages$LibPath))),
              "\n\t", paste("This environment DOES ", ifelse(object@exclude.site, "NOT ", ""), "combine with the current site library location when loaded.", sep=""),
              "\n\n"))
})

##' switchBack
##'
##' A convenience function to switch back to the previously used computing
##' environment.
##' @export
switchBack = function() {
    if(length(Renvs$stack) < 2) {
        warning("No previous computing environment to switch back to. Computing environment will remain unchanged")
        return(NULL)
    }
    switchTo(Renvs$stack[[2]], reverting=TRUE)
}

##' currentCompEnv
##'
##' Display the computing environment currently in use. If switchTo has not been
##' called, a new SwitchrCtx object describing the current environment is
##' created.
##' @export   
currentCompEnv = function() {
            if(is.null(Renvs$stack)) {
                lp = .libPaths()
                lp = lp[!(lp %in% .Library | lp %in% .Library.site)]
                Renvs$stack = list(original = SwitchrCtx("original",
                                       libpaths = lp , seed = NULL,
                                       exclude.site=FALSE))
            }
            Renvs$stack[[1]]
        }


globalVariables(".lib.loc")

.libPaths2 = function(fulllp, exclude.site=TRUE) {
    fun = .libPaths
    lst = list()
    lst$.Library.site = if(exclude.site) character() else .Library.site
    
    environment(fun) = list2env(lst,
                   parent = environment(.libPaths))
    fun(fulllp)
}


    
